//
// Fragment shader for computing vertex positions & normals for R2VB water rendering demo
//

uniform	sampler3D	noiseMap;
uniform	float		time;
uniform float		waveScale;

vec3	turbulence ( const in vec3 p, const in float freqScale )
{
	float	sum = 0.0;
	vec3	t   = vec3 ( 0.0 );
	float	f   = 1.0;

	for ( int i = 0; i <= 3; i++ )
	{
		t   += abs ( 2.0 * texture3D ( noiseMap, f * p.xyz ).rgb - vec3 ( 1.0 ) ) / f;
		sum += 1.0 / f;
		f   *= freqScale;
	}

										// remap from [0,sum] to [-1,1]
	return 2.0 * t / sum - vec3 ( 1.0 );
}

void	main ()
{										// deltas
	const vec3	dx = vec3 ( 0.01, 0.0,  0.0 );
	const vec3	dy = vec3 ( 0.0,  0.01, 0.0 );
	
										// compute height and delta height (in dx and dy)
	vec3	tex    = vec3       ( gl_TexCoord [0].xy, 0.1*time );
	vec3	turb   = turbulence ( tex,      2.0 );
	vec3	turbX  = turbulence ( tex + dx, 2.0 );
	vec3	turbY  = turbulence ( tex + dy, 2.0 );
	vec3	n      = vec3       ( turbX.x - turb.x, turbY.x - turb.x, 1.0 );
	
	gl_FragData [0] = vec4 ( 20.0*gl_TexCoord [0].xy, waveScale * turb.x, 1.0 );
	gl_FragData [1] = vec4 ( normalize ( n ), 1.0 );
}
