//
// Fragment shader for computing vertex positions for skeletal animation
//

uniform	sampler2DRect	vertDataMap;
uniform	sampler2DRect	weightsMap;
uniform	sampler2DRect	jointsMap;

//
// Quaternion multiplication
//

vec4	quatMul ( in vec4 q1, in vec4 q2 )
{
	vec3	im = q1.w * q2.xyz + q1.xyz * q2.w + cross ( q1.xyz, q2.xyz );
	vec4	dt = q1 * q2;
	float	re = dot ( dt, vec4 ( -1.0, -1.0, -1.0, 1.0 ) );
	
	return vec4 ( im, re );
}

//
// vector rotation via quaternion
//

vec4	quatRotate ( in vec3 p, in vec4 q )
{
	vec4	temp = quatMul ( q, vec4 ( p, 0.0 ) );
	
	return quatMul ( temp, vec4 ( -q.x, -q.y, -q.z, q.w ) );
}

vec3	boneTransf ( in vec4 bonePosParent, in vec4 boneOrient, vec3 pos )
{
	return bonePosParent.xyz + quatRotate ( pos, boneOrient ).xyz;
}

//
// Get couple of vec4 values from textureRect
//

void	readVec8 ( in sampler2DRect s, in int index, out vec4 v1, out vec4 v2 )
{
	int	index2 = 2 * index;
	int	y      = int ( index2 / 512 );
	int	x      = index2 - 512 * y;
	int	y2     = y;
	int	x2     = x + 1;
	
	if ( x2 >= 512 )
	{
		y2 ++;
		x2 -= 512;
	}
	
	v1 = texture2DRect ( s, vec2 ( x,  y  ) );
	v2 = texture2DRect ( s, vec2 ( x2, y2 ) );
}

void	main ()
{
	vec2	weights     = texture2DRect ( vertDataMap, gl_TexCoord [0].xy ).xy;
	int		startWeight = int ( weights.x );
	int		weightCount = int ( weights.y );
	vec3	pos         = vec3 ( 0.0 );
	vec4	pw, jz;												// (pos.xyz, weight), (jointIndex, 0, 0, 0)
	vec4	pp, orient;											// (pos.xyz, parent), (orient.xyzw)
	
	for ( int i = 0; i < weightCount; i++ )
	{
		int	index = startWeight + i;
		
		readVec8 ( weightsMap, startWeight + i, pw, jz );
		readVec8 ( jointsMap, int ( jz.x ), pp, orient );
		
		pos += boneTransf ( pp, orient, pw.xyz ) * pw.w;
	}

	gl_FragColor = vec4 ( pos.yxz, 1 );
}